@@ -4,10 +4,11 @@ between R objects handled by rpy2 and pandas objects."""
 import rpy2.robjects.conversion as conversion
 import rpy2.rinterface as rinterface
 from rpy2.rinterface_lib import na_values
-from rpy2.rinterface import (IntSexpVector,
-                             Sexp,
-                             SexpVector,
-                             StrSexpVector)
+from rpy2.rinterface import IntSexpVector
+from rpy2.rinterface import ListSexpVector
+from rpy2.rinterface import SexpVector
+from rpy2.rinterface import StrSexpVector
+
 import datetime
 import functools
 import math
@@ -23,6 +24,7 @@ from rpy2.robjects.vectors import (BoolVector,
                                    DataFrame,
                                    DateVector,
                                    FactorVector,
+                                   FloatVector,
                                    FloatSexpVector,
                                    StrVector,
                                    IntVector,
@@ -53,7 +55,7 @@ def py2rpy_pandasdataframe(obj):
     od = OrderedDict()
     for name, values in obj.iteritems():
         try:
-            od[name] = conversion.py2rpy(values)
+            od[name] = conversion.converter_ctx.get().py2rpy(values)
         except Exception as e:
             warnings.warn('Error while trying to convert '
                           'the column "%s". Fall back to string conversion. '
@@ -93,19 +95,26 @@ def py2rpy_categoryseries(obj):
     return res
 
 
-def _str_populate_r_vector(iterable, r_vector,
-                           set_elt,
-                           cast_value):
-    for i, v in enumerate(iterable):
-        if (
-                v is None
-                or
-                v is pandas.NA
-                or
-                (isinstance(v, float) and math.isnan(v))
-        ):
-            v = na_values.NA_Character
-        set_elt(r_vector, i, cast_value(v))
+def _populate_r_vector_with_na(na_value):
+    def f(iterable, r_vector,
+          set_elt,
+          cast_value):
+        for i, v in enumerate(iterable):
+            if (
+                    v is None
+                    or
+                    v is pandas.NA
+                    or
+                    (isinstance(v, float) and math.isnan(v))
+            ):
+                v = na_value
+            set_elt(r_vector, i, cast_value(v))
+    return f
+
+
+_str_populate_r_vector = _populate_r_vector_with_na(na_values.NA_Character)
+_bool_populate_r_vector = _populate_r_vector_with_na(na_values.NA_Logical)
+_float_populate_r_vector = _populate_r_vector_with_na(na_values.NA_Real)
 
 
 def _int_populate_r_vector(iterable, r_vector,
@@ -123,12 +132,19 @@ _PANDASTYPE2RPY2 = {
         IntVector.from_iterable,
         populate_func=_int_populate_r_vector
     ),
-    bool: BoolVector,
+    pandas.BooleanDtype: functools.partial(
+        BoolVector.from_iterable,
+        populate_func=_bool_populate_r_vector
+    ),
     None: BoolVector,
     str: functools.partial(
         StrVector.from_iterable,
         populate_func=_str_populate_r_vector
     ),
+    pandas.Float64Dtype: functools.partial(
+        FloatVector.from_iterable,
+        populate_func=_float_populate_r_vector
+    ),
     bytes: (numpy2ri.converter.py2rpy.registry[
         numpy.ndarray
     ])
@@ -157,7 +173,7 @@ def py2rpy_pandasseries(obj):
         # TODO: can the POSIXct be created from the POSIXct constructor ?
         # (is '<M8[ns]' mapping to Python datetime.datetime ?)
         res = POSIXct(res)
-    elif obj.dtype.type == str:
+    elif obj.dtype.type is str:
         res = _PANDASTYPE2RPY2[str](obj)
     elif obj.dtype.name in integer_array_types:
         res = _PANDASTYPE2RPY2[int](obj)
@@ -185,6 +201,8 @@ def py2rpy_pandasseries(obj):
                     (homogeneous_type, type(x)))
         # TODO: Could this be merged with obj.type.name == 'O' case above ?
         res = _PANDASTYPE2RPY2[homogeneous_type](obj)
+    elif type(obj.dtype) in (pandas.Float64Dtype, pandas.BooleanDtype):
+        res = _PANDASTYPE2RPY2[type(obj.dtype)](obj)
     else:
         # converted as a numpy array
         func = numpy2ri.converter.py2rpy.registry[numpy.ndarray]
@@ -202,7 +220,8 @@ def py2rpy_pandasseries(obj):
                            StrVector(tuple(str(x) for x in obj.index)))
     else:
         res.do_slot_assign('dimnames',
-                           SexpVector(conversion.py2rpy(obj.index)))
+                           SexpVector(conversion.converter_ctx
+                                      .get().py2rpy(obj.index)))
     return res
 
 
@@ -226,11 +245,77 @@ def rpy2py_posixct(obj):
                               errors='coerce')
 
 
+def _records_to_columns(obj):
+    columns = OrderedDict()
+    obj_ri = ListSexpVector(obj)
+    # First pass to get the column names.
+    for i, record in enumerate(obj_ri):
+        checknames = set()
+        for name in record.names:
+            if name in checknames:
+                raise ValueError(
+                    f'The record {i} has "{name}" duplicated.'
+                )
+            checknames.add(name)
+            if name not in columns:
+                columns[name] = []
+    columnnames = set(columns.keys())
+    # Second pass to fill the columns.
+    for i, record in enumerate(obj_ri):
+        checknames = set()
+        for name, value in zip(record.names, record):
+            checknames.add(name)
+            if hasattr(value, '__len__'):
+                if len(value) != 1:
+                    raise ValueError(
+                        f'The value for "{name}" record {i} is not a scalar. '
+                        f'It has {len(value)} elements.'
+                    )
+                else:
+                    value = value[0]
+            columns[name].append(value)
+        # Set to NA/None missing column values in the record.
+        for name in columnnames-checknames:
+            columns[name].append(None)
+    return columns
+
+
+def _flatten_dataframe(obj, colnames_lst):
+    """Make each element in a list of columns or group of
+    columns iterable.
+
+    This is an helper function to make the "flattening" of columns
+    in an R data frame easier as each item in the top-level iterable
+    can be a column or list or records (themselves each with an
+    arbitrary number of columns).
+
+    Args:
+    - colnames_list: an *empty* list that will be populated with
+    column names.
+    """
+    for i, n in enumerate(obj.colnames):
+        col = obj[i]
+        if isinstance(col, ListSexpVector):
+            _ = _records_to_columns(col)
+            colnames_lst.extend((n, subn) for subn in _.keys())
+            for subcol in _.values():
+                yield subcol
+        else:
+            colnames_lst.append(n)
+            yield col
+
+
 @rpy2py.register(DataFrame)
 def rpy2py_dataframe(obj):
-    items = OrderedDict((k, rpy2py(v) if isinstance(v, Sexp) else v)
-                        for k, v in obj.items())
-    res = PandasDataFrame.from_dict(items)
+    rpy2py = conversion.get_conversion().rpy2py
+    colnames_lst = []
+    od = OrderedDict(
+        (i, rpy2py(col) if isinstance(col, rinterface.SexpVector) else col)
+        for i, col in enumerate(_flatten_dataframe(obj, colnames_lst))
+    )
+
+    res = pandas.DataFrame.from_dict(od)
+    res.columns = tuple('.'.join(_) if isinstance(_, list) else _ for _ in colnames_lst)
     res.index = obj.rownames
     return res
 
@@ -252,7 +337,7 @@ converter._rpy2py_nc_map.update(
             {'factor': _to_pandas_factor}
         ),
         rinterface.ListSexpVector: conversion.NameClassMap(
-            numpy2ri.rpy2py,
+            numpy2ri.rpy2py_list,
             {'data.frame': lambda obj: rpy2py(DataFrame(obj))}
         )
     }
@@ -271,10 +356,10 @@ def activate():
 
     original_converter = conversion.Converter(
         'snapshot before pandas conversion',
-        template=conversion.converter)
+        template=conversion.get_conversion())
     numpy2ri.activate()
     new_converter = conversion.Converter('snapshot before pandas conversion',
-                                         template=conversion.converter)
+                                         template=conversion.get_conversion())
     numpy2ri.deactivate()
 
     for k, v in py2rpy.registry.items():
